// proext.c
// Extract AltAcc datafiles from a Palm OS database

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "palmdb.h"

//----------------------------------------------------------------------------

unsigned short swap2 (unsigned short x)

{
	return (x & 0xFF00) >> 8 | (x << 8);
}

//----------------------------------------------------------------------------

unsigned int swap4 (unsigned int x)

{
	return (x & 0xFF000000) >> 24 | (x & 0x00FF0000) >> 8 |
			(x & 0x0000FF00) << 8 | (x << 24);
}

//----------------------------------------------------------------------------
// make an 8.3 filename

const char* MakeFilename (const char* desc, const char* ext)

{
	static char filename[32];
	char*		s = filename;
	const char*	p;
	
	for (p = desc; *p; p++) {
		if (isalnum (*p)) 
			*(s++) = *p;

		// check len
		if (s - filename == 8)
			break;
	}

	// Terminate string
	*s = '\0';

	// add extension
	strcat (filename, ext);

	return filename;
}

//----------------------------------------------------------------------------

int main (int argc, const char* argv[])

{
	FILE*		fp;
	FILE*		fout;
	PDBHeader	header;
	PDBRecord*	reclist;
	ProReadData	data;
	const char*	filename;
	int			i, numrecs, recsize;

	// Check usage
	if (argc < 2) {
		printf ("usage: proext DBFile\n");
		exit (-1);
	}

	// Try to open the file
	fp = fopen (argv[1], "rb");
	if (!fp) {
		printf ("unable to open Palm DB file \"%s\"\n", argv[1]);
		exit (1);
	}

	if (fread (&header, sizeof (PDBHeader), 1, fp) < 1) {
		printf ("error reading file header\n");
		exit (1);
	}

	printf ("Name           - %s\n", header.Name);
	printf ("Attributes     - %04x\n", swap2 (header.Attr));
	printf ("Number of Recs - %d\n\n", swap2 (header.NumberOfRecords));

	// Get the number of records
	numrecs = swap2 (header.NumberOfRecords);

	// Determine the size of the record list
	recsize = numrecs * sizeof (PDBRecord);

	// read the record list array
	reclist = (PDBRecord*) malloc (recsize);
	if (!reclist) {
		printf ("error allocating reclist\n");
		exit (1);
	}

	if (fread (reclist, recsize, 1, fp) < 1) {
		printf ("error reading record list\n");
		exit (1);
	}

	for (i = 0; i < numrecs; i++) {
		// Get the records
		fseek (fp, swap4 (reclist[i].DataOffset), 0);
		if (fread (&data, sizeof (ProReadData), 1, fp) < 1) {
			printf ("error reading record %d\n", i);
			exit (1);
		}

		// Open a file to write the altacc data to
		filename = MakeFilename (data.name, ".dat");
		fout = fopen (filename, "wb");
		if (!fout) {
			printf ("error opening %s\n", filename);
			exit (1);
		}

		//write the data
		if (fwrite (data.data, sizeof (data.data), 1, fout) < 1){
			printf ("Error writing %s\n", filename);
			exit (1);
		}

		fclose (fout);

		// Log progress
		printf ("%3d %02x ", i, reclist[i].Attributes);
		printf ("%-32s %12s\n", data.name, filename);
	}

	fclose (fp);

	return 0;
}